﻿function GetPluginSettings()
{
	return {
		"name":			"Lottie",
		"id":			"VinoosLottie",
		"version":		"5.5.9",
		"description":	"Lottie is a library for Android, iOS, Web, and Windows that parses Adobe After Effects animations exported as json with Bodymovin and renders them natively on mobile and on the web!",
		"author":		"MilaDesign",
		"help url":		"http://www.vinoos.ir/help/plugins/lottie.php",
		"category":		"Vinoos",
		"dependency":	"lottie.min.js;lottie_api.min.js",
		"type":			"world",
		"rotatable":	true,
		"flags":		pf_position_aces | pf_size_aces | pf_angle_aces | pf_zorder_aces
	};
};

////////////////////////////////////////
// Conditions
AddCondition(0, cf_trigger, "On Data Ready", "Lottie", "On data is ready", "Triggered when all parts of the animation have been loaded.", "onDataReady");
AddCondition(1, cf_trigger, "On Complete", "Lottie", "On animation complete", "Triggered when animation completed.", "onComplete");
AddCondition(2, cf_trigger, "On Loop Complete", "Lottie", "On animation loop complete", "Triggered when animation loop completed.", "onLoopComplete");
AddCondition(4, cf_trigger, "On Enter Frame", "Lottie", "On enter frame", "Triggered when frame changed.", "onEnterFrame");
AddCondition(3, cf_trigger, "On Clicked", "Lottie", "On view clicked", "Triggered when view clicked.", "onClicked");

////////////////////////////////////////
// Actions
AddAction(0, af_none, "Play", "Lottie", "Play animation", "Start playing animation", "Play");
AddAction(1, af_none, "Stop", "Lottie", "Stop animation", "Stop playing animation", "Stop");
AddAction(2, af_none, "Pause", "Lottie", "Pause animation", "pause the animation", "Pause");

AddComboParamOption("Invisible");
AddComboParamOption("Visible");
AddComboParam("Visibility", "Choose whether to hide or show the view.");
AddAction(3, af_none, "Set Visible", "Appearance", "Set <b>{0}</b>", "Hide or show the view.", "SetVisible");

AddNumberParam("Speed");
AddAction(4, af_none, "Set Speed", "Lottie", "Set speed to <b>{0}</b>", "Change animation speed.", "SetSpeed");

AddNumberParam("Value");
AddComboParamOption("False");
AddComboParamOption("True");
AddComboParam("Is Frame", "Time or frames.");
AddAction(5, af_none, "Go To And Play", "Lottie", "Go to <b>{0}</b> and play", "Go to and play.", "GoToAndPlay");

AddNumberParam("Value");
AddComboParamOption("False");
AddComboParamOption("True");
AddComboParam("Is Frame", "Time or frames.");
AddAction(6, af_none, "Go To And Stop", "Lottie", "Go to <b>{0}</b> and stop", "Go to and stop.", "GoToAndStop");

AddNumberParam("Direction");
AddAction(7, af_none, "Set Direction", "Lottie", "Set direction to <b>{0}</b>", "Change animation direction.", "SetDirection");

////////////////////////////////////////
// Expressions
AddExpression(0, ef_return_number, "Returns current loop.", "Lottie", "CurrentLoop", "Returns current loop.");
AddExpression(1, ef_return_number, "Returns current time.", "Lottie", "CurrentTime", "Returns current time.");
AddExpression(2, ef_return_number, "Returns total time.", "Lottie", "TotalTime", "Returns total time.");

ACESDone();

// Property grid properties for this plugin
var property_list = [
	new cr.Property(ept_combo,	"Loop",					"Yes",		"Choose whether the button is enabled or disabled on startup.", "No|Yes"),
	new cr.Property(ept_combo,	"Autoplay",				"Yes",		"Automatically set the font size depending on the layer scale.", "No|Yes"),
	new cr.Property(ept_text,	"FileName",				"",		"An ID for the control allowing it to be styled with CSS from the page HTML."),
	new cr.Property(ept_combo,	"Initial visibility",	"Visible",	"Choose whether the button is visible on startup.", "Invisible|Visible"),
	];
	
// Called by IDE when a new object type is to be created
function CreateIDEObjectType()
{
	return new IDEObjectType();
}

// Class representing an object type in the IDE
function IDEObjectType()
{
	assert2(this instanceof arguments.callee, "Constructor called as a function");
}

// Called by IDE when a new object instance of this type is to be created
IDEObjectType.prototype.CreateInstance = function(instance)
{
	return new IDEInstance(instance);
}

// Class representing an individual instance of an object in the IDE
function IDEInstance(instance, type)
{
	assert2(this instanceof arguments.callee, "Constructor called as a function");
	
	// Save the constructor parameters
	this.instance = instance;
	this.type = type;
	
	// Set the default property values from the property table
	this.properties = {};
	
	for (var i = 0; i < property_list.length; i++)
		this.properties[property_list[i].name] = property_list[i].initial_value;
		
	// Plugin-specific variables
	this.just_inserted = false;
	this.font = null;
}

IDEInstance.prototype.OnCreate = function()
{
	this.instance.SetHotspot(new cr.vector2(0, 0));
}

IDEInstance.prototype.OnInserted = function()
{
	this.instance.SetSize(new cr.vector2(96, 96));
}

IDEInstance.prototype.OnDoubleClicked = function()
{
}

// Called by the IDE after a property has been changed
IDEInstance.prototype.OnPropertyChanged = function(property_name)
{
}

IDEInstance.prototype.OnRendererInit = function(renderer)
{
}
	
// Called to draw self in the editor
IDEInstance.prototype.Draw = function(renderer)
{
	renderer.SetTexture(null);
	var quad = this.instance.GetBoundingQuad();
	renderer.Fill(quad, cr.RGB(224, 224, 224));
	renderer.Outline(quad, cr.RGB(0, 0, 0));
	
	if (!this.font)
		this.font = renderer.CreateFont("Arial", 14, false, false);
	
	var text = "Lottie\r\n" + this.properties["FileName"]
	
	this.font.DrawText(text,
						this.instance.GetBoundingQuad(),
						cr.RGB(0, 0, 0),
						ha_center,
						this.instance.GetOpacity(),
						this.instance.GetAngle(),
						false,
						0,
						ha_center);
}

IDEInstance.prototype.OnRendererReleased = function(renderer)
{
	this.font = null;
}