﻿// ECMAScript 5 strict mode
"use strict";

assert2(cr, "cr namespace not created");
assert2(cr.plugins_, "cr.plugins_ not created");

/////////////////////////////////////
// Plugin class
cr.plugins_.VinoosLottie = function(runtime)
{
	this.runtime = runtime;
};

(function ()
{
	/////////////////////////////////////
	var pluginProto = cr.plugins_.VinoosLottie.prototype;
		
	/////////////////////////////////////
	// Object type class
	pluginProto.Type = function(plugin)
	{
		this.plugin = plugin;
		this.runtime = plugin.runtime;
	};

	var typeProto = pluginProto.Type.prototype;

	var currentLoop = 0;
	var currentTime = 0;
	var totalTime = 0;
	
	// called on startup for each object type
	typeProto.onCreate = function()
	{
	};

	/////////////////////////////////////
	// Instance class
	pluginProto.Instance = function(type)
	{
		this.type = type;
		this.runtime = type.runtime;
	};
	
	var instanceProto = pluginProto.Instance.prototype;
	
	// called whenever an instance is created
	instanceProto.onCreate = function()
	{
		// Not supported in directCanvas
		if (this.runtime.isDomFree)
		{
			cr.logexport("[Construct 2] VinoosLottie plugin not supported on this platform - the object will not be created");
			return;
		}
		
		var self = this;
		this.anim;
		
		this.div = document.createElement("div");
		
		this.loop = (this.properties[0] !== 0);
		this.autoplay = (this.properties[1] !== 0);
		this.filename = this.properties[2];
		
		jQuery(this.div).appendTo(this.runtime.canvasdiv ? this.runtime.canvasdiv : "body");
		
		var params = {
			container: this.div,
			renderer: 'svg',
			loop: this.loop,
			autoplay: this.autoplay,
			path: this.filename + '.json'
		};
		
		this.anim = lottie.loadAnimation(params);
		
		
		if (this.properties[3] === 0)
		{
			jQuery(this.div).hide();
			this.visible = false;
			this.element_hidden = true;
		}
		
		this.div.onclick = (function (self) {
			return function(e) {
				e.stopPropagation();
				
				self.runtime.isInUserInputEvent = true;
				self.runtime.trigger(cr.plugins_.VinoosLottie.prototype.cnds.onClicked, self);
				self.runtime.isInUserInputEvent = false;
			};
		})(this);
		
		this.anim.addEventListener('data_ready',function(){
			self.runtime.trigger(cr.plugins_.VinoosLottie.prototype.cnds.onDataReady, self);
		});
		
		this.anim.addEventListener('complete',function(data){
			self.runtime.trigger(cr.plugins_.VinoosLottie.prototype.cnds.onComplete, self);
		});
		
		this.anim.addEventListener('loopComplete',function(data){
			currentLoop = data.currentLoop;
			self.runtime.trigger(cr.plugins_.VinoosLottie.prototype.cnds.onLoopComplete, self);
		});
		
		this.anim.addEventListener('enterFrame',function(data){
			currentTime = data.currentTime;
			totalTime = data.totalTime;
			self.runtime.trigger(cr.plugins_.VinoosLottie.prototype.cnds.onEnterFrame, self);
		});
		
		this.lastLeft = 0;
		this.lastTop = 0;
		this.lastRight = 0;
		this.lastBottom = 0;
		this.lastWinWidth = 0;
		this.lastWinHeight = 0;
			
		this.updatePosition(true);
		
		this.runtime.tickMe(this);
	};
	
	instanceProto.saveToJSON = function ()
	{
		var o = {
			"loop": this.loop,
			"autoplay": !!this.autoplay
		};
		
		return o;
	};
	
	instanceProto.loadFromJSON = function (o)
	{
		this.loop = o["loop"];
		this.autoplay = o["autoplay"];
	};
	
	instanceProto.onDestroy = function ()
	{
		if (this.runtime.isDomFree)
			return;
		
		jQuery(this.div).remove();
		this.div = null;
	};
	
	instanceProto.tick = function ()
	{
		this.updatePosition();
	};
	
	var last_canvas_offset = null;
	var last_checked_tick = -1;
	
	instanceProto.updatePosition = function (first)
	{
		if (this.runtime.isDomFree)
			return;
		
		var left = this.layer.layerToCanvas(this.x, this.y, true);
		var top = this.layer.layerToCanvas(this.x, this.y, false);
		var right = this.layer.layerToCanvas(this.x + this.width, this.y + this.height, true);
		var bottom = this.layer.layerToCanvas(this.x + this.width, this.y + this.height, false);
		
		var rightEdge = this.runtime.width / this.runtime.devicePixelRatio;
		var bottomEdge = this.runtime.height / this.runtime.devicePixelRatio;
		
		// Is entirely offscreen or invisible: hide
		if (!this.visible || !this.layer.visible || right <= 0 || bottom <= 0 || left >= rightEdge || top >= bottomEdge)
		{
			if (!this.element_hidden)
				jQuery(this.div).hide();
			
			this.element_hidden = true;
			return;
		}
		
		// Truncate to canvas size
		if (left < 1)
			left = 1;
		if (top < 1)
			top = 1;
		if (right >= rightEdge)
			right = rightEdge - 1;
		if (bottom >= bottomEdge)
			bottom = bottomEdge - 1;
		
		var curWinWidth = window.innerWidth;
		var curWinHeight = window.innerHeight;
			
		// Avoid redundant updates
		if (!first && this.lastLeft === left && this.lastTop === top && this.lastRight === right && this.lastBottom === bottom && this.lastWinWidth === curWinWidth && this.lastWinHeight === curWinHeight)
		{
			if (this.element_hidden)
			{
				jQuery(this.div).show();
				this.element_hidden = false;
			}
			
			return;
		}
			
		this.lastLeft = left;
		this.lastTop = top;
		this.lastRight = right;
		this.lastBottom = bottom;
		this.lastWinWidth = curWinWidth;
		this.lastWinHeight = curWinHeight;
		
		if (this.element_hidden)
		{
			jQuery(this.div).show();
			this.element_hidden = false;
		}
		
		var offx = Math.round(left) + jQuery(this.runtime.canvas).offset().left;
		var offy = Math.round(top) + jQuery(this.runtime.canvas).offset().top;
		jQuery(this.div).css("position", "absolute");
		jQuery(this.div).offset({left: offx, top: offy});
		jQuery(this.div).width(Math.round(right - left));
		jQuery(this.div).height(Math.round(bottom - top));
		
		if (this.angle > 0) {
			jQuery(this.div).css("transform", "rotate(" + this.angle + "rad)");
		}
	};
	
	// only called if a layout object
	instanceProto.draw = function(ctx)
	{
	};
	
	instanceProto.drawGL = function(glw)
	{
	};
	
	/**BEGIN-PREVIEWONLY**/
	instanceProto.getDebuggerValues = function (propsections)
	{
		propsections.push({
			"title": "VinoosLottie",
			"properties": [
				{"name": "Loop", "value": this.loop},
				{"name": "Autoplay", "value": this.autoplay}
			]
		});
	};
	
	instanceProto.onDebugValueEdited = function (header, name, value)
	{
		if (name === "Loop")
		{
			this.loop = value;
		} else if (name === "Autoplay")
		{
			this.autoplay = value;
		}
	};
	/**END-PREVIEWONLY**/

	//////////////////////////////////////
	// Conditions
	function Cnds() {};
	
	Cnds.prototype.onDataReady = function () {
		return true;
	};
	
	Cnds.prototype.onComplete = function () {
		return true;
	};
	
	Cnds.prototype.onLoopComplete = function () {
		return true;
	};
	
	Cnds.prototype.onEnterFrame = function () {
		return true;
	};
	
	Cnds.prototype.onClicked = function () {
		return true;
	};
	
	pluginProto.cnds = new Cnds();
	
	//////////////////////////////////////
	// Actions
	function Acts() {};

	Acts.prototype.Play = function ()
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.play();
	};
	
	Acts.prototype.Stop = function ()
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.stop();
	};
	
	Acts.prototype.Pause = function ()
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.pause();
	};
	
	Acts.prototype.SetVisible = function (vis)
	{
		if (this.runtime.isDomFree)
			return;
		
		this.visible = (vis !== 0);
	};
	
	Acts.prototype.SetSpeed = function (speed)
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.setSpeed(speed);
	};
	
	Acts.prototype.SetDirection = function (direction)
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.setDirection(direction);
	};
	
	Acts.prototype.GoToAndPlay = function (value, isFrame)
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.goToAndPlay(value, (isFrame !== 0));
	};
	
	Acts.prototype.GoToAndStop = function (value, isFrame)
	{
		if (this.runtime.isDomFree)
			return;
		
		this.anim.goToAndStop(value, (isFrame !== 0));
	};
	
	pluginProto.acts = new Acts();
	
	//////////////////////////////////////
	// Expressions
	function Exps() {};

	Exps.prototype.CurrentLoop = function (ret) {
		ret.set_int(currentLoop);
	};
	
	Exps.prototype.CurrentTime = function (ret) {
		ret.set_int(currentTime);
	};
	
	Exps.prototype.TotalTime = function (ret) {
		ret.set_int(totalTime);
	};
	
	pluginProto.exps = new Exps();

}());